import argparse
import os
from paffi import load_topology, get_nodes_keys, get_node_latitude, get_node_longitude, get_node_attr, \
    get_delay, get_delays, get_delay_source, get_delay_destination


def scale_coords(lat, lon, bb):
    minlat, maxlat, minlon, maxlon = bb
    y = (1 - (lat - minlat) / (maxlat - minlat)) * 500
    x = (lon - minlon) / (maxlon - minlon) * 500
    return x, y


def get_bb(topology, nodetype):
    minlat = -1
    maxlat = -1
    minlon = -1
    maxlon = -1
    keys = get_nodes_keys(topology, nodetype)
    for i in keys:
        lat = get_node_latitude(topology, nodetype, i)
        lon = get_node_longitude(topology, nodetype, i)
        if minlat < 0 or lat < minlat:
            minlat = lat
        if maxlat < 0 or lat > maxlat:
            maxlat = lat
        if minlon < 0 or lon < minlon:
            minlon = lon
        if maxlon < 0 or lon > maxlon:
            maxlon = lon
    return minlat, maxlat, minlon, maxlon


# FIXME: should add a .json or a .ini file with the configuration for omnet
parser = argparse.ArgumentParser()
parser.add_argument("-i", "--input", help="Scenario file. Default: scenario.json")
parser.add_argument("-o", "--output", help="Prefix for .ned and .ini file. Default: <input filename>")
parser.add_argument("-n", "--netname", help="Network name for Oment simulation. Default: Fog<Input_file>")
parser.add_argument("-t", "--template", help="Prefix for template .ned and .ini file. Default: omnet_resources/scenario_template")
args = parser.parse_args()

initemplate = "%s.ini" % args.template if args.template else "omnet_resources/scenario_template.ini"
nedtemplate = "%s.ned" % args.template if args.template else "omnet_resources/scenario_template.ned"

infile = args.input if args.template else "scenario.json"
outbase = args.output if args.output else os.path.splitext(infile)[0]
iniout = "%s.ini" % args.output if args.output else "%s.ini" % outbase
nedout = "%s.ned" % args.output if args.output else "%s.ned" % outbase

# load data
topology = load_topology(infile)

if args.netname:
    netname = args.netname
else:
    netname = "Fog%s" % outbase.capitalize()

# FIXME: connections are going to change structure
connections_sf = ''
conntype = 'conn_sf'

for i in get_delays(topology, conntype):
    connections_sf += '\t\tsource[%s].dataout --> {delay = %f s;} --> fog[%s].in++;\n' % \
                      (get_delay_source(topology, delaytype=conntype, lbl=i),
                       get_delay(topology, delaytype=conntype, lbl=i),
                       get_delay_destination(topology, delaytype=conntype, lbl=i))

connections_fc = ''
conntype = 'conn_fc'
for i in get_delays(topology, conntype):
    connections_fc += '\t\tfog[%s].dataout --> {delay = %f s;} --> cloud[%s].in++;\n' % \
                      (get_delay_source(topology, delaytype=conntype, lbl=i),
                       get_delay(topology, delaytype=conntype, lbl=i),
                       get_delay_destination(topology, delaytype=conntype, lbl=i))

# connectionsff = ''
# for a in AD['Delay FF']:
#    connectionsFF += '\t\tfog[' + str(a['ID Fog 1']) + '].toNeighbors++ --> {delay = ' + str(
#        a['Delay']) + 'ms;} --> fog[' + str(a['ID Fog 2']) + '].fromNeighbors++;\n'

nsources = str(len(get_nodes_keys(topology, 'sensors')))
nfog = str(len(get_nodes_keys(topology, 'fog')))
ncloud = str(len(get_nodes_keys(topology, 'cloud')))

sendintervals = ''
for i in get_nodes_keys(topology, 'sensors'):
    sendintervals += '%s.source[%s].interArrivalTime = exponential(1s / %s)\n' % \
                     (netname, i, get_node_attr(topology, 'sensors', i, 'lambda'))

servicetimes = ''
for i in get_nodes_keys(topology, 'fog'):
    servicetimes += '%s.fog[%s].serviceTime = exponential(1s / %s)\n' % \
                    (netname, i, get_node_attr(topology, 'fog', i, 'mu'))

bb = get_bb(topology, 'sensors')

positions = ''
for i in get_nodes_keys(topology, 'sensors'):
    x, y = scale_coords(get_node_latitude(topology, 'sensors', i), get_node_longitude(topology, 'sensors', i), bb)
    positions += '%s.source[%s].xpos = %f\n%s.source[%s].ypos = %f\n' % (netname, i, x, netname, i, y)

for i in get_nodes_keys(topology, 'fog'):
    x, y = scale_coords(get_node_latitude(topology, 'fog', i), get_node_longitude(topology, 'fog', i), bb)
    positions += '%s.fog[%s].xpos = %f\n%s.fog[%s].ypos = %f\n' % (netname, i, x, netname, i, y)

for i in get_nodes_keys(topology, 'cloud'):
    x, y = scale_coords(get_node_latitude(topology, 'cloud', i), get_node_longitude(topology, 'cloud', i), bb)
    positions += '%s.cloud[%s].xpos = %f\n%s.cloud[%s].ypos = %f\n' % (netname, i, x, netname, i, y)

to_replace_ned = {"*CONNECTIONSSF*": connections_sf,
                  "*CONNECTIONSFC*": connections_fc,
                  "*NUMSOURCE*": nsources,
                  "*NUMFOG*": nfog,
                  "*NUMCLOUD*": ncloud,
                  "*NETNAME*": netname
                  }
# "*CONNECTIONSFF*": connections_ff,

# creating .ned file from template
with open(nedtemplate, 'r') as tfile:
    template = tfile.readlines()

with open(nedout, "w+") as ttemp:
    for line in template:
        for k in to_replace_ned:
            line = line.replace(k, to_replace_ned[k])
        ttemp.write(line)

to_replace_ini = {"*POSITIONS*": positions,
                  "*SERVICETIMES*": servicetimes,
                  "*SENDINTERVALS*": sendintervals,
                  "*NETNAME*": netname
                  }

# creating .ini file from template

with open(initemplate, 'r') as tfile:
    template = tfile.readlines()

with open(iniout, 'w+') as ttemp:
    for line in template:
        for k in to_replace_ini:
            line = line.replace(k, to_replace_ini[k])
        ttemp.write(line)
