import json
import argparse
from paffi import create_topology, set_node, save_topology

parser = argparse.ArgumentParser()
parser.add_argument("-t", "--type", help="Type of files: [sensors|fog|cloud]. Default = sensors")
parser.add_argument("-i", "--input", help="Input txt file to use. Default = POI_<type>.txt")
parser.add_argument("-o", "--output", help="Output file. Default = <type>.json")

args = parser.parse_args()


def isfloat(s):
    try:
        float(s)
    except ValueError:
        return False
    return True


def valid(t):
    # tuple format: ('name', 'latitude', longitude, 'neighborhood name')
    try:
        if t[0].isnumeric():
            return False
        if not isfloat(t[1]):
            return False
        if not isfloat(t[2]):
            return False
        if t[3].isnumeric():
            return False
    except IndexError:
        return False
    # check if latitude and longitude are OK
    if (float(t[1]) < -90.0) or (float(t[1]) > 90.0):
        return False
    if (float(t[2]) < -180.0) or (float(t[2]) > 180.0):
        return False
    return True


def check(line):
    line = line.rstrip("\n")
    line = line.split("\t")
    if len(line) == 4:
        if valid(line):
            return {"address": line[0], "lat": line[1], "lon": line[2], "neighborhood": line[3]}
    return None


nodetype = args.type if args.type else "sensors"
fin = args.input if args.input else "POI_%s.txt" % nodetype
fout = args.output if args.output else "%s.json" % nodetype
nodes = {}

with open(fin, 'r') as f:
    i = 0
    for l in f:
        v = check(l)
        if v is not None:
            nodes[i] = v
            i += 1

topology = create_topology()
for i in nodes:
    set_node(topology, nodetype, i, lat=nodes[i]['lat'], lon=nodes[i]['lon'], address=nodes[i]['address'])

save_topology(topology, fout)
