from paffi.topology import get_nodes_keys, get_node_attr, set_connection, set_connection_type, get_connection_label, \
    get_delays, get_delay, get_delay_source, get_delay_destination


def parse_ampl_id(w):
    if w.startswith("s") or w.startswith("f") or w.startswith("c"):
            return int(w[1:])
    else:
        return None


def read_ampl_solution(ampl_out, variable):
    solution = {}
    compformat = True
    with open(ampl_out, "r") as f:
        for line in f:
            if variable in line:
                break
        for line in f:
            if ";" in line:
                break
            if line.startswith(":"):
                compformat = False
            else:
                ww = line.split()
                if compformat:
                    id1 = parse_ampl_id(ww[0])
                    id2 = parse_ampl_id(ww[1])
                    solution[str(id1)] = id2
                else:
                    id = parse_ampl_id(ww[0])
                    solution[str(id)] = ww.index("1") - 1
    return solution


def connect_ampl(topology, ampl_out):
    set_connection_type(topology, 'ampl')
    solution = read_ampl_solution(ampl_out, "X[s,f]")
    for i in solution:
        conntype = 'conn_sf'
        set_connection(topology, conntype=conntype, src=i, dst=solution[i])
    solution = read_ampl_solution(ampl_out, "Y[f,c]")
    for i in solution:
        conntype = 'conn_fc'
        set_connection(topology, conntype=conntype, src=i, dst=solution[i])


def save_set(fout, setname, setprefix, set):
    # sets of sensors
    out = "set %s :=" % setname
    for i in set:
        out = out + (" %s%s" % (setprefix, i))
    out = out + ";\n"
    # print(dataout)
    fout.write(out)


def save_param(topology, nodetype, fout, pname, setprefix, param):
    out = "param %s :=\n" % pname
    for i in get_nodes_keys(topology, nodetype):
        out = out + ("\t%s%s\t%s\n" % (setprefix, i, get_node_attr(topology, nodetype, i, param)))
    out = out + ";\n"
    # print(dataout)
    fout.write(out)


def save_distance(topology, delaytype, fout, pname, setprefix1, setprefix2):
    out = "param %s :=\n" % pname
    for i in get_delays(topology, delaytype):
        out = out + ("\t%s%s\t%s%s\t%f\n" % (setprefix1, get_delay_source(topology, delaytype=delaytype, lbl=i),
                                             setprefix2, get_delay_destination(topology, delaytype=delaytype, lbl=i),
                                             get_delay(topology, delaytype=delaytype, lbl=i)))
    out = out + ";\n"
    # print(dataout)
    fout.write(out)


def write_ampl_data(out_filename, topology):
    with open("%s.dat" % out_filename, "w") as fout:
        # dump data
        # sets of sources, fog nodes, cloud
        save_set(fout, "S", "s", topology["sensors"])
        save_set(fout, "F", "f", topology["fog"])
        save_set(fout, "C", "c", topology["cloud"])
        # parameters lambda, mu
        save_param(topology, 'sensors', fout, 'lambda', "s", 'lambda')
        save_param(topology, 'fog', fout, 'mu', "f", 'mu')
        # source-to-fog and fog-to-cloud distances
        save_distance(topology, 'delay_sf', fout, 'distance_sf', 's', 'f')
        save_distance(topology, 'delay_fc', fout, 'distance_fc', 'f', 'c')

        # save_distance(fout, "distance_fc", "f", "c", topology["delay_fc"])
